/****************************************************************************
**
** Copyright (C) 2005-2006 Trolltech ASA. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include <QTableWidget>
#include <QHeaderView>
#include <QGridLayout>
#include <QBrush>
#include <QColor>
#include "usredit.h"
#include "glwidget.h"


// Globals
extern QString zhuLocale;
extern QTranslator zhuTranslator;
extern OGLWidget *oglWid;


// Constructor
usrWidget::usrWidget(QWidget *parent) : QWidget(parent) {

	setWindowTitle(tr("User defined items"));
	readSettings();
	createUsrTable();

	QGridLayout *mainLayout = new QGridLayout;
	mainLayout->addWidget(usrTable, 0, 0);
	setLayout(mainLayout);

	// Cell changed
	connect(usrTable, SIGNAL(cellChanged(int, int)), this, SLOT(entryChanged(const int , const int)));

	// A fun-edi input was erroneous. This is fetched in oglWid
	connect(oglWid, SIGNAL(parsersTainted(bool)), this, SLOT(lockUsrSlot(bool)));
}


// Row count starts with 0 in good old C-tradition
void usrWidget::entryChanged(const int r, const int c) {

	int err;
	static bool whiteMessage=false;
	static bool redMessage=false;
	static bool selMessage=false;
	QBrush brush(QColor(255,255,255));

	// This caused some headache:-) Changing colours emit a cellChanged event.
	// Same for error case. So avoid multiple recursions
	if(whiteMessage || selMessage)
		return;

	// Avoid recursions on the next deepest level too
	if(redMessage) {
		for(int i=0; i<MAXUSR; i++) {
			selMessage=true;
			usrTable->item(i,c)->setFlags(Qt::ItemIsSelectable);
			selMessage=false;
		}
		selMessage=true;
		usrTable->item(r,c)->setFlags(Qt::ItemIsEnabled | Qt::ItemIsEditable);
		selMessage=false;
		return;
	}

	// Fetch new entry
	usrErr[r]=false;
	usr[r]=usrTable->item(r,c)->text();

	// Rebuild all parsers with error check
	if((err=initAllParsers()) != FunctionParser::FP_NO_ERROR) {
		QString message=fpError::getQtMessage(err)+"\n";
		QMessageBox::critical(this, tr("Alert"), message);
		usrErr[r]=true;
		brush.setColor(QColor(255,127,127));
		redMessage=true;
		usrTable->item(r,c)->setBackground(brush);
		redMessage=false;
		emit parsersTainted(true);
		return;
	}

	// Now all entries MUST be ok
	for(int i=0; i<MAXUSR; i++) {
		selMessage=true;
		usrTable->item(i,c)->setFlags(Qt::ItemIsEnabled | Qt::ItemIsEditable);
		selMessage=false;
	}

	// Update on success
	brush.setColor(QColor(255,255,255));
	whiteMessage=true;
	usrTable->item(r,c)->setBackground(brush);
	whiteMessage=false;
	emit parsersTainted(false);
	iniITL();
	usrTable->setCurrentCell(r+1,c);
	oglWid->gotNewItem();
}


void usrWidget::deleteUsrTable() {
	for(int i=0; i<MAXUSR; i++)
		delete usrTable->item(i,0);
	delete usrTable;
}


void usrWidget::createUsrTable() {
	usrTable = new QTableWidget(0, 1);
	QStringList columnLabel(tr("Item"));

	usrTable->setHorizontalHeaderLabels(columnLabel);
	usrTable->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
	usrTable->verticalHeader()->show();
	usrTable->setShowGrid(true);

	for(int i=0; i<MAXUSR; i++) {
		QTableWidgetItem *entryItem = new QTableWidgetItem(usr[i]);
		entryItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsEditable);
		usrTable->insertRow(i);
		usrTable->setItem(i, 0, entryItem);
	}
}


/* *************************************************** */
/* REIMPLEMENT EVENTS FOR LOCALIZATIONS                */
/* *************************************************** */
void usrWidget::changeEvent(QEvent* event) {
	if(event->type() == QEvent::LanguageChange) {
		setWindowTitle(tr("User defined items"));
		QStringList columnLabel(tr("Item"));
		usrTable->setHorizontalHeaderLabels(columnLabel);
	}
	else
		QWidget::changeEvent(event);
}


/* *************************************************** */
/* HANDLE SAVE/RESTORE OF WINDOW-COORDINATES           */
/* *************************************************** */
void usrWidget::closeEvent(QCloseEvent *event) {
	writeSettings();
	event->accept();
}

void usrWidget::readSettings() {
	QSettings settings("Heinz", NAME);
	QPoint pos=settings.value("UsrWindowPos", QPoint(200, 200)).toPoint();
	QSize size=settings.value("UsrWindowSize", QSize(400, 400)).toSize();

	// Ensure to start with defaults on 1st program-start ever
	everStarted=false;
	everStarted=settings.value("FirstUsrEdiStart").toBool();
	if(everStarted) {
		resize(size);
		move(pos);
	}
	else {
		everStarted=true;
		resize(QSize(165, 280));
		move(QPoint(100, 330));
	}
}

void usrWidget::writeSettings() {
	QSettings settings("Heinz", NAME);
	settings.setValue("UsrWindowPos", pos());
	settings.setValue("UsrWindowSize", size());
	settings.setValue("FirstUsrEdiStart", everStarted);
}
